@@ -1,5 +1,10 @@
 #' @importFrom stats nobs
-.compute_variances <- function(x, component, name_fun = NULL, name_full = NULL, verbose = TRUE) {
+.compute_variances <- function(x,
+                               component,
+                               name_fun = NULL,
+                               name_full = NULL,
+                               verbose = TRUE,
+                               tolerance = 1e-5) {
 
   ## Original code taken from GitGub-Repo of package glmmTMB
   ## Author: Ben Bolker, who used an cleaned-up/adapted
@@ -10,13 +15,9 @@
 
   faminfo <- model_info(x)
 
-  if (!faminfo$is_mixed) {
-    stop("Model is not a mixed model.", call. = FALSE)
-  }
-
-  if (faminfo$family %in% c("truncated_nbinom1", "truncated_nbinom2")) {
+  if (faminfo$family %in% c("truncated_nbinom1")) {
     if (verbose) {
-      warning(sprintf("Truncated negative binomial families are currently not supported by `%s`.", name_fun), call. = F)
+      warning(sprintf("Truncated negative binomial families are currently not supported by `%s`.", name_fun), call. = FALSE)
     }
     return(NA)
   }
@@ -27,9 +28,9 @@
 
   # Test for non-zero random effects ((near) singularity)
   no_random_variance <- FALSE
-  if (.is_singular(x, vals) && !(component %in% c("slope", "intercept"))) {
+  if (.is_singular(x, vals, tolerance = tolerance) && !(component %in% c("slope", "intercept"))) {
     if (verbose) {
-      warning(sprintf("Can't compute %s. Some variance components equal zero.\n  Solution: Respecify random structure!", name_full), call. = F)
+      warning(sprintf("Can't compute %s. Some variance components equal zero. Your model may suffer from singulariy.\n  Solution: Respecify random structure!\n  You may also decrease the 'tolerance' level to enforce the calculation of random effect variances.", name_full), call. = FALSE)
     }
     no_random_variance <- TRUE
   }
@@ -43,6 +44,7 @@
   var.intercept <- NULL
   var.slope <- NULL
   cor.slope_intercept <- NULL
+  cor.slopes <- NULL
 
   # Get variance of fixed effects: multiply coefs by design matrix
   if (component %in% c("fixed", "all")) {
@@ -60,7 +62,7 @@
   obs.terms <- names(nr[nr == n_obs(x)])
 
   # Variance of random effects
-  if (component %in% c("random", "all") && !isTRUE(no_random_variance)) {
+  if (component %in% c("random", "all") && isFALSE(no_random_variance)) {
     var.random <- .compute_variance_random(not.obs.terms, x = x, vals = vals)
   }
 
@@ -68,11 +70,22 @@
   # additive dispersion and the distribution-specific variance (Johnson et al. 2014)
 
   if (component %in% c("residual", "distribution", "all")) {
-    var.distribution <- .compute_variance_distribution(x, var.cor = vals$vc, faminfo, name = name_full, verbose = verbose)
+    var.distribution <- .compute_variance_distribution(
+      x = x,
+      var.cor = vals$vc,
+      faminfo,
+      name = name_full,
+      verbose = verbose
+    )
   }
 
   if (component %in% c("residual", "dispersion", "all")) {
-    var.dispersion <- .compute_variance_dispersion(x = x, vals = vals, faminfo = faminfo, obs.terms = obs.terms)
+    var.dispersion <- .compute_variance_dispersion(
+      x = x,
+      vals = vals,
+      faminfo = faminfo,
+      obs.terms = obs.terms
+    )
   }
 
   if (component %in% c("residual", "all")) {
@@ -91,6 +104,10 @@
     cor.slope_intercept <- .random_slope_intercept_corr(vals, x)
   }
 
+  if (component %in% c("all")) {
+    cor.slopes <- .random_slopes_corr(vals, x)
+  }
+
   # if we only need residual variance, we can delete those
   # values again...
   if (component == "residual") {
@@ -107,7 +124,8 @@
     "var.dispersion" = var.dispersion,
     "var.intercept" = var.intercept,
     "var.slope" = var.slope,
-    "cor.slope_intercept" = cor.slope_intercept
+    "cor.slope_intercept" = cor.slope_intercept,
+    "cor.slopes" = cor.slopes
   ))
 }
 
@@ -132,6 +150,7 @@
     stop("Package `rstanarm` needs to be installed to compute variances for mixed models.", call. = FALSE)
   }
 
+  # stanreg
   if (inherits(x, "stanreg")) {
     vals <- list(
       beta = lme4::fixef(x),
@@ -139,6 +158,8 @@
       vc = lme4::VarCorr(x),
       re = lme4::ranef(x)
     )
+
+    # GLMMapdative
   } else if (inherits(x, "MixMod")) {
     vals <- list(
       beta = lme4::fixef(x),
@@ -147,11 +168,12 @@
       re = list(lme4::ranef(x))
     )
     names(vals$re) <- x$id_name
+
+    # nlme
   } else if (inherits(x, "lme")) {
     re_names <- find_random(x, split_nested = TRUE, flatten = TRUE)
-    comp_x <- as.matrix(cbind(`(Intercept)` = 1, get_predictors(x)))
+    comp_x <- stats::model.matrix(x, data = get_data(x))
     rownames(comp_x) <- 1:nrow(comp_x)
-
     if (.is_nested_lme(x)) {
       vals_vc <- .get_nested_lme_varcorr(x)
       vals_re <- lme4::ranef(x)
@@ -159,16 +181,16 @@
       vals_vc <- list(nlme::getVarCov(x))
       vals_re <- list(lme4::ranef(x))
     }
-
     vals <- list(
       beta = lme4::fixef(x),
       X = comp_x,
       vc = vals_vc,
       re = vals_re
     )
-
     names(vals$re) <- re_names
     names(vals$vc) <- re_names
+
+    # ordinal
   } else if (inherits(x, "clmm")) {
     if (requireNamespace("ordinal", quietly = TRUE)) {
       f <- find_formula(x)$conditional
@@ -180,6 +202,8 @@
         re = ordinal::ranef(x)
       )
     }
+
+    # glmmadmb
   } else if (inherits(x, "glmmadmb")) {
     vals <- list(
       beta = lme4::fixef(x),
@@ -187,6 +211,46 @@
       vc = lme4::VarCorr(x),
       re = lme4::ranef(x)
     )
+
+    # brms
+  } else if (inherits(x, "brmsfit")) {
+    # comp_x <- as.matrix(cbind(`(Intercept)` = 1, get_predictors(x)))
+    formula_rhs <- .safe_deparse(find_formula(x)$conditional[[3]])
+    formula_rhs <- stats::as.formula(paste0("~", formula_rhs))
+    comp_x <- stats::model.matrix(formula_rhs, data = get_data(x))
+    rownames(comp_x) <- 1:nrow(comp_x)
+    vc <- lapply(names(lme4::VarCorr(x)), function(i) {
+      element <- lme4::VarCorr(x)[[i]]
+      if (i != "residual__") {
+        if (!is.null(element$cov)) {
+          out <- as.matrix(drop(element$cov[, 1, ]))
+          colnames(out) <- rownames(out) <- gsub("Intercept", "(Intercept)", rownames(element$cov), fixed = TRUE)
+        } else {
+          out <- as.matrix(drop(element$sd[, 1])^2)
+          colnames(out) <- rownames(out) <- gsub("Intercept", "(Intercept)", rownames(element$sd), fixed = TRUE)
+        }
+        attr(out, "sttdev") <- element$sd[, 1]
+      } else {
+        out <- NULL
+      }
+      out
+    })
+    vc <- .compact_list(vc)
+    names(vc) <- setdiff(names(lme4::VarCorr(x)), "residual__")
+    attr(vc, "sc") <- lme4::VarCorr(x)$residual__$sd[1, 1]
+    vals <- list(
+      beta = lme4::fixef(x)[, 1],
+      X = comp_x,
+      vc = vc,
+      re = lapply(lme4::ranef(x), function(re) {
+        reval <- as.data.frame(drop(re[, 1, ]))
+        colnames(reval) <- gsub("Intercept", "(Intercept)", dimnames(re)[[3]], fixed = TRUE)
+        reval
+      })
+    )
+    names(vals$beta) <- gsub("Intercept", "(Intercept)", names(vals$beta), fixed = TRUE)
+
+    # cpglmm
   } else if (inherits(x, "cpglmm")) {
     if (!requireNamespace("cplm", quietly = TRUE)) {
       stop("Package 'cplm' required. Please install it.")
@@ -197,6 +261,8 @@
       vc = cplm::VarCorr(x),
       re = cplm::ranef(x)
     )
+
+    # lme4 / glmmTMB
   } else {
     vals <- list(
       beta = lme4::fixef(x),
@@ -206,6 +272,7 @@
     )
   }
 
+
   # for glmmTMB, tell user that dispersion model is ignored
 
   if (inherits(x, "glmmTMB")) {
@@ -247,8 +314,11 @@
 
 
 
-# Get fixed effects variance
-#
+
+
+
+# fixed effects variance ----
+
 #' @importFrom stats var
 .compute_variance_fixed <- function(vals) {
   with(vals, stats::var(as.vector(beta %*% t(X))))
@@ -258,10 +328,15 @@
 
 
 
-# Compute variance associated with a random-effects term (Johnson 2014)
-#
+
+
+# variance associated with a random-effects term (Johnson 2014) ----
+
 #' @importFrom stats nobs
 .compute_variance_random <- function(terms, x, vals) {
+  if (is.null(terms)) {
+    return(NULL)
+  }
   .sigma_sum <- function(Sigma) {
     rn <- rownames(Sigma)
 
@@ -269,7 +344,7 @@
       valid <- rownames(Sigma) %in% colnames(vals$X)
       if (!all(valid)) {
         rn <- rn[valid]
-        Sigma <- Sigma[valid, valid]
+        Sigma <- Sigma[valid, valid, drop = FALSE]
       }
     }
 
@@ -288,7 +363,10 @@
 
 
 
-# Calculate Distribution-specific variance (Nakagawa et al. 2017)
+
+
+# distribution-specific variance (Nakagawa et al. 2017) ----
+
 .compute_variance_distribution <- function(x, var.cor, faminfo, name, verbose = TRUE) {
   if (inherits(x, "lme")) {
     sig <- x$sigma
@@ -304,7 +382,16 @@
   if (faminfo$is_linear && !faminfo$is_tweedie) {
     dist.variance <- sig^2
   } else {
-    if (faminfo$is_binomial) {
+    if (faminfo$is_betabinomial) {
+      dist.variance <- switch(
+        faminfo$link_function,
+        logit = ,
+        probit = ,
+        cloglog = ,
+        clogloglink = .variance_distributional(x, faminfo, sig, name = name, verbose = verbose),
+        .badlink(faminfo$link_function, faminfo$family, verbose = verbose)
+      )
+    } else if (faminfo$is_binomial) {
       dist.variance <- switch(
         faminfo$link_function,
         logit = pi^2 / 3,
@@ -321,12 +408,13 @@
         .badlink(faminfo$link_function, faminfo$family, verbose = verbose)
       )
     } else if (faminfo$family %in% c("Gamma", "gamma")) {
-      ## TODO needs some more checking
+      ## TODO needs some more checking - should now be in line with other packages
       dist.variance <- switch(
         faminfo$link_function,
         inverse = ,
-        identity = stats::family(x)$variance(sig),
-        log = .variance_distributional(x, faminfo, sig, name = name, verbose = verbose),
+        identity = ,
+        log = stats::family(x)$variance(sig),
+        # log = .variance_distributional(x, faminfo, sig, name = name, verbose = verbose),
         .badlink(faminfo$link_function, faminfo$family, verbose = verbose)
       )
     } else if (faminfo$family == "beta") {
@@ -352,7 +440,9 @@
 
 
 
-# Get dispersion-specific variance
+
+# dispersion-specific variance ----
+
 .compute_variance_dispersion <- function(x, vals, faminfo, obs.terms) {
   if (faminfo$is_linear) {
     0
@@ -368,6 +458,8 @@
 
 
 
+
+
 # This is the core-function to calculate the distribution-specific variance
 # Nakagawa et al. 2017 propose three different methods, here we only rely
 # on the lognormal-approximation.
@@ -382,17 +474,17 @@
   # see Nakagawa et al. 2017
 
   # in general want log(1+var(x)/mu^2)
-  null_model <- .null_model(x, verbose = verbose)
+  .null_model <- null_model(x, verbose = verbose)
 
   # check if null-model could be computed
-  if (!is.null(null_model)) {
-    if (inherits(null_model, "cpglmm")) {
+  if (!is.null(.null_model)) {
+    if (inherits(.null_model, "cpglmm")) {
       if (!requireNamespace("cplm", quietly = TRUE)) {
         stop("Package 'cplm' required. Please install it.")
       }
-      null_fixef <- unname(cplm::fixef(null_model))
+      null_fixef <- unname(cplm::fixef(.null_model))
     } else {
-      null_fixef <- unname(.collapse_cond(lme4::fixef(null_model)))
+      null_fixef <- unname(.collapse_cond(lme4::fixef(.null_model)))
     }
     mu <- exp(null_fixef)
   } else {
@@ -401,13 +493,13 @@
 
   if (is.na(mu)) {
     if (verbose) {
-      warning("Can't calculate model's distribution-specific variance. Results are not reliable.", call. = F)
+      warning("Can't calculate model's distribution-specific variance. Results are not reliable.", call. = FALSE)
     }
     return(0)
   }
   else if (mu < 6) {
     if (verbose) {
-      warning(sprintf("mu of %0.1f is too close to zero, estimate of %s may be unreliable.\n", mu, name), call. = FALSE)
+      warning(sprintf("mu of %0.1f is too close to zero, estimate of %s may be unreliable.", mu, name), call. = FALSE)
     }
   }
 
@@ -416,37 +508,43 @@
       vv <- switch(
         faminfo$family,
 
-        # (zero-inflated) poisson
+        # (zero-inflated) poisson ----
         `zero-inflated poisson` = ,
         poisson = .variance_family_poisson(x, mu, faminfo),
 
-        # hurdle-poisson
+        # hurdle-poisson ----
         `hurdle poisson` = ,
         truncated_poisson = stats::family(x)$variance(sig),
 
-        # Gamma, exponential
+        # Gamma, exponential ----
         Gamma = stats::family(x)$variance(sig),
 
-        # (zero-inflated) negative binomial
+        # (zero-inflated) negative binomial ----
         `zero-inflated negative binomial` = ,
         `negative binomial` = ,
         genpois = ,
         nbinom1 = ,
         nbinom2 = .variance_family_nbinom(x, mu, sig, faminfo),
+        truncated_nbinom2 = stats::family(x)$variance(mu, sig),
 
-        # other distributions
+        # other distributions ----
         tweedie = .variance_family_tweedie(x, mu, sig),
         beta = .variance_family_beta(x, mu, sig),
+        # betabinomial = stats::family(x)$variance(mu, sig),
+        # betabinomial = .variance_family_betabinom(x, mu, sig),
 
-        # default variance for non-captured distributions
+        # default variance for non-captured distributions ----
         .variance_family_default(x, mu, verbose)
       )
 
+      if (vv < 0 && isTRUE(verbose)) {
+        warning("Model's distribution-specific variance is negative. Results are not reliable.", call. = FALSE)
+      }
       vv / mu^2
     },
     error = function(x) {
       if (verbose) {
-        warning("Can't calculate model's distribution-specific variance. Results are not reliable.", call. = F)
+        warning("Can't calculate model's distribution-specific variance. Results are not reliable.", call. = FALSE)
       }
       0
     }
@@ -488,8 +586,20 @@
 
 
 
+# Get distributional variance for beta-family
+.variance_family_betabinom <- function(x, mu, phi) {
+  if (inherits(x, "MixMod")) {
+    stats::family(x)$variance(mu)
+  } else {
+    n <- n_obs(x)
+    mu * (1 - mu) * (n * (phi + n) / (1 + phi))
+  }
+}
+
+
+
+
 # Get distributional variance for tweedie-family
-#
 #' @importFrom stats plogis
 .variance_family_tweedie <- function(x, mu, phi) {
   p <- unname(stats::plogis(x$fit$par["thetaf"]) + 1)
@@ -554,7 +664,7 @@
   # pearson residuals
   # pred <- predict(model, type = "response") ## (1 - p) * mu
   # pred <- stats::predict(model, type_pred = "response", type = "mean_subject")
-  # (insight::get_response(model) - pred) / sqrt(pvar)
+  # (get_response(model) - pred) / sqrt(pvar)
 }
 
 
@@ -596,13 +706,20 @@
         mu * (1 + mu / lme4::getME(x, "glmer.nb.theta"))
       } else if (inherits(x, "MixMod")) {
         stats::family(x)$variance(mu)
+      } else if (inherits(x, "glmmTMB")) {
+        if (is.null(x$theta)) {
+          theta <- lme4::getME(x, "theta")
+        } else {
+          theta <- x$theta
+        }
+        mu * (1 + mu / theta)
       } else {
         mu * (1 + mu / x$theta)
       }
     },
     error = function(x) {
       if (verbose) {
-        warning("Can't calculate model's distribution-specific variance. Results are not reliable.", call. = F)
+        warning("Can't calculate model's distribution-specific variance. Results are not reliable.", call. = FALSE)
       }
       0
     }
@@ -612,51 +729,6 @@
 
 
 
-# Null model is needed to calculate the mean for the model's response,
-# which we need to compute the distribution-specific variance
-# (see .variance_distributional())
-#
-#' @importFrom stats as.formula update reformulate
-.null_model <- function(model, verbose = TRUE) {
-  if (!requireNamespace("lme4", quietly = TRUE)) {
-    stop("Package `lme4` needs to be installed to compute variances for mixed models.", call. = FALSE)
-  }
-
-  if (inherits(model, "MixMod")) {
-    nullform <- stats::as.formula(paste(find_response(model), "~ 1"))
-    null.model <- stats::update(model, fixed = nullform)
-  } else if (inherits(model, "cpglmm")) {
-    nullform <- find_formula(model)[["random"]]
-    null.model <- stats::update(model, nullform)
-  } else {
-    f <- stats::formula(model)
-    resp <- find_response(model)
-    re.terms <- paste0("(", sapply(lme4::findbars(f), .safe_deparse), ")")
-    nullform <- stats::reformulate(re.terms, response = resp)
-    null.model <- tryCatch(
-      {
-        stats::update(model, nullform)
-      },
-      error = function(e) {
-        msg <- e$message
-        if (verbose) {
-          if (grepl("(^object)(.*)(not found$)", msg)) {
-            insight::print_color("Can't calculate null-model. Probably the data that was used to fit the model cannot be found.\n", "red")
-          } else if (grepl("^could not find function", msg)) {
-            insight::print_color("Can't calculate null-model. Probably you need to load the package that was used to fit the model.\n", "red")
-          }
-        }
-        return(NULL)
-      }
-    )
-  }
-
-  null.model
-}
-
-
-
-
 # return existence of random slopes
 .random_slopes_in_fixed <- function(model) {
   rs <- find_random_slopes(model)
@@ -668,6 +740,11 @@
     return(TRUE)
   }
 
+  # NULL models have no predictors, so no fixed effect as random slope
+  if (is.null(fe)) {
+    return(FALSE)
+  }
+
   # make sure we have identical subcomponents between random and
   # fixed effects
   fe <- .compact_list(fe[c("conditional", "zero_inflated")])
@@ -733,3 +810,37 @@
     unlist(rho01)
   }
 }
+
+
+
+# slope-slope-correlations (rho 01)
+#' @importFrom utils combn
+#' @importFrom stats setNames
+.random_slopes_corr <- function(vals, x) {
+  corrs <- lapply(vals$vc, attr, "correlation")
+  rnd_slopes <- unlist(find_random_slopes(x))
+
+  if (length(rnd_slopes) < 2) {
+    return(NULL)
+  }
+
+  rho01 <- tryCatch(
+    {
+      sapply(corrs, function(i) {
+        if (!is.null(i)) {
+          slope_pairs <- utils::combn(x = rnd_slopes, m = 2, simplify = FALSE)
+          lapply(slope_pairs, function(j) {
+            stats::setNames(i[j[1], j[2]], paste0("..", paste0(j, collapse = "-")))
+          })
+        } else {
+          NULL
+        }
+      })
+    },
+    error = function(e) {
+      NULL
+    }
+  )
+
+  unlist(rho01)
+}
