@@ -31,6 +31,13 @@
 #' data(mtcars)
 #' m <- lm(mpg ~ wt + cyl + vs, data = mtcars)
 #' find_formula(m)
+#'
+#' if (require("lme4")) {
+#'   m <- lmer(Sepal.Length ~ Sepal.Width + (1 | Species), data = iris)
+#'   f <- find_formula(m)
+#'   f
+#'   format(f)
+#' }
 #' @importFrom stats formula terms as.formula
 #' @export
 find_formula <- function(x, ...) {
@@ -44,12 +51,7 @@ find_formula <- function(x, ...) {
 
 #' @export
 find_formula.default <- function(x, ...) {
-  if (inherits(x, "list") && .obj_has_name(x, "gam")) {
-    x <- x$gam
-    class(x) <- c(class(x), c("glm", "lm"))
-  }
-
-  tryCatch(
+  f <- tryCatch(
     {
       list(conditional = stats::formula(x))
     },
@@ -57,6 +59,29 @@ find_formula.default <- function(x, ...) {
       NULL
     }
   )
+  .find_formula_return(f)
+}
+
+
+
+#' @export
+find_formula.list <- function(x, ...) {
+  if (.obj_has_name(x, "gam")) {
+    if ("mer" %in% names(x)) {
+      f.random <- .fix_gamm4_random_effect(find_formula(x$mer)$random)
+      if (length(f.random) == 1) {
+        f.random <- f.random[[1]]
+      } else if (length(f.random) == 0) {
+        f.random <- NULL
+      }
+    }
+    x <- x$gam
+    class(x) <- c(class(x), c("glm", "lm"))
+    f <- .compact_list(list(conditional = stats::formula(x), random = f.random))
+  } else {
+    f <- find_formula.default(x, ...)
+  }
+  .find_formula_return(f)
 }
 
 
@@ -72,11 +97,16 @@ find_formula.data.frame <- function(x, ...) {
 find_formula.aovlist <- function(x, ...) {
   f <- attr(x, "terms", exact = TRUE)
   attributes(f) <- NULL
-  list(conditional = f)
+  .find_formula_return(list(conditional = f))
 }
 
 
 
+#' @export
+find_formula.anova <- function(x, ...) {
+  stop("Formulas cannot be retrieved from anova() objects.")
+}
+
 
 
 
@@ -114,19 +144,16 @@ find_formula.gam <- function(x, ...) {
     }
   }
 
-  f
+  .find_formula_return(f)
 }
 
 
 
 #' @export
 find_formula.gamlss <- function(x, ...) {
-  tryCatch(
+  f <- tryCatch(
     {
-      if (!requireNamespace("lme4", quietly = TRUE)) {
-        stop("To use this function, please install package 'lme4'.")
-      }
-      f.random <- lapply(lme4::findbars(x$mu.formula), function(.x) {
+      f.random <- lapply(.findbars(x$mu.formula), function(.x) {
         f <- .safe_deparse(.x)
         stats::as.formula(paste0("~", f))
       })
@@ -147,6 +174,7 @@ find_formula.gamlss <- function(x, ...) {
       NULL
     }
   )
+  .find_formula_return(f)
 }
 
 
@@ -156,19 +184,41 @@ find_formula.gamlss <- function(x, ...) {
 find_formula.bamlss <- function(x, ...) {
   f <- stats::formula(x)
 
-  .compact_list(list(
-    conditional = stats::as.formula(.safe_deparse(f$mu$formula)),
-    sigma = stats::as.formula(paste0("~", as.character(f$sigma$formula)[3]))
+  if (!is.null(f$mu)) {
+    f.cond <- f$mu$formula
+  } else if (!is.null(f$pi)) {
+    f.cond <- f$pi$formula
+  }
+
+  if (!is.null(f$sigma)) {
+    f.sigma <- stats::as.formula(paste0("~", as.character(f$sigma$formula)[3]))
+  } else if (!is.null(f$pi)) {
+    f.sigma <- NULL
+  }
+
+  f <- .compact_list(list(
+    conditional = stats::as.formula(.safe_deparse(f.cond)),
+    sigma = f.sigma
   ))
+  .find_formula_return(f)
 }
 
 
 
 #' @export
 find_formula.gamm <- function(x, ...) {
-  x <- x$gam
-  class(x) <- c(class(x), c("glm", "lm"))
-  NextMethod()
+  f <- .compact_list(find_formula(x$gam))
+  random <- .fix_gamm_random_effect(names(x$lme$groups))
+
+  if (length(random) == 0) {
+    f.random <- NULL
+  } else if (length(random) > 1) {
+    f.random <- lapply(random, function(r) stats::as.formula(paste0("~1|", r)))
+  } else {
+    f.random <- stats::as.formula(paste0("~1|", random))
+  }
+
+  .find_formula_return(.compact_list(c(f, list(random = f.random))))
 }
 
 
@@ -176,12 +226,119 @@ find_formula.gamm <- function(x, ...) {
 
 
 
+# Meta-Analysis -----------------------
+
+
+#' @export
+find_formula.rma <- function(x, ...) {
+  NULL
+}
+
+#' @export
+find_formula.metaplus <- find_formula.rma
+
+#' @export
+find_formula.meta_random <- find_formula.rma
+
+#' @export
+find_formula.meta_fixed <- find_formula.rma
+
+#' @export
+find_formula.meta_bma <- find_formula.rma
+
+
+
+
+
+
+
 # Other models ----------------------------------------------
 
 
+#' @export
+find_formula.censReg <- find_formula.default
+
+#' @export
+find_formula.maxLik <- find_formula.default
+
+#' @export
+find_formula.maxim <- find_formula.default
+
+
+#' @export
+find_formula.mediate <- function(x, ...) {
+  f <- list(
+    mediator = find_formula(x$model.m),
+    outcome = find_formula(x$model.y)
+  )
+  .find_formula_return(f)
+}
+
+
+#' @export
+find_formula.averaging <- function(x, ...) {
+  f_random <- tryCatch(
+    {
+      models <- attributes(x)$modelList
+      find_formula(models[[1]])
+    },
+    error = function(e) {
+      NULL
+    }
+  )
+
+  f <- find_formula.default(x)
+  if (!.obj_has_name(f, "random") && .obj_has_name(f_random, "random")) {
+    f$random <- f_random$random
+  }
+
+  .find_formula_return(f)
+}
+
+
+#' @export
+find_formula.glht <- function(x, ...) {
+  .find_formula_return(list(conditional = stats::formula(x$model)))
+}
+
+
+#' @export
+find_formula.betareg <- function(x, ...) {
+  f <- stats::formula(x)
+  fs <- .safe_deparse(f)
+
+  if (grepl("|", fs, fixed = TRUE)) {
+    fs <- trimws(unlist(strsplit(fs, "|", fixed = TRUE)))
+    f <- list(
+      conditional = stats::as.formula(fs[1]),
+      precision = stats::as.formula(paste0("~", fs[2]))
+    )
+  } else {
+    f <- list(conditional = f)
+  }
+  .find_formula_return(f)
+}
+
+
+#' @export
+find_formula.afex_aov <- function(x, ...) {
+  if ("aov" %in% names(x)) {
+    find_formula(x$aov)
+  } else {
+    find_formula(x$lm)
+  }
+}
+
+
+#' @export
+find_formula.mira <- function(x, ...) {
+  .find_formula_return(find_formula(x$analyses[[1]]))
+}
+
+
 #' @export
 find_formula.gee <- function(x, ...) {
-  tryCatch(
+  f <- tryCatch(
     {
       id <- parse(text = .safe_deparse(x$call))[[1]]$id
 
@@ -198,16 +355,18 @@ find_formula.gee <- function(x, ...) {
       NULL
     }
   )
+  .find_formula_return(f)
 }
 
 
 
 #' @export
 find_formula.MANOVA <- function(x, ...) {
-  .compact_list(list(
+  f <- .compact_list(list(
     conditional = x$input$formula,
     random = stats::as.formula(paste0("~", x$input$subject))
   ))
+  .find_formula_return(f)
 }
 
 #' @export
@@ -218,7 +377,17 @@ find_formula.RM <- find_formula.MANOVA
 #' @export
 find_formula.gls <- function(x, ...) {
   ## TODO this is an intermediate fix to return the correlation variables from gls-objects
-  f_corr <- parse(text = .safe_deparse(x$call$correlation))[[1]]$form
+  fcorr <- x$call$correlation
+  if (!is.null(fcorr)) {
+    f_corr <- parse(text = .safe_deparse(x$call$correlation))[[1]]
+  } else {
+    f_corr <- NULL
+  }
+  if (is.symbol(f_corr)) {
+    f_corr <- paste("~", .safe_deparse(f_corr))
+  } else {
+    f_corr <- f_corr$form
+  }
 
   l <- tryCatch(
     {
@@ -232,14 +401,14 @@ find_formula.gls <- function(x, ...) {
     }
   )
 
-  .compact_list(l)
+  .find_formula_return(.compact_list(l))
 }
 
 
 
 #' @export
 find_formula.LORgee <- function(x, ...) {
-  tryCatch(
+  f <- tryCatch(
     {
       id <- parse(text = .safe_deparse(x$call))[[1]]$id
 
@@ -256,13 +425,14 @@ find_formula.LORgee <- function(x, ...) {
       NULL
     }
   )
+  .find_formula_return(f)
 }
 
 
 
 #' @export
 find_formula.cglm <- function(x, ...) {
-  tryCatch(
+  f <- tryCatch(
     {
       id <- parse(text = .safe_deparse(x$call))[[1]]$id
 
@@ -279,6 +449,7 @@ find_formula.cglm <- function(x, ...) {
       NULL
     }
   )
+  .find_formula_return(f)
 }
 
 
@@ -286,12 +457,52 @@ find_formula.cglm <- function(x, ...) {
 
 
 
+
+# mfx models ---------------------------------------
+
+
+#' @export
+find_formula.betamfx <- find_formula.betareg
+
+#' @export
+find_formula.betaor <- find_formula.betareg
+
+#' @export
+find_formula.logitmfx <- function(x, ...) {
+  find_formula.default(x$fit, ...)
+}
+
+#' @export
+find_formula.poissonmfx <- find_formula.logitmfx
+
+#' @export
+find_formula.negbinmfx <- find_formula.logitmfx
+
+#' @export
+find_formula.logitor <- find_formula.logitmfx
+
+#' @export
+find_formula.negbinirr <- find_formula.logitmfx
+
+#' @export
+find_formula.poissonirr <- find_formula.logitmfx
+
+#' @export
+find_formula.probitmfx <- find_formula.logitmfx
+
+
+
+
+
+
+
+
 # Panel data models ---------------------------------------
 
 
 #' @export
 find_formula.ivreg <- function(x, ...) {
-  tryCatch(
+  f <- tryCatch(
     {
       f <- .safe_deparse(stats::formula(x))
       cond <- .trim(substr(f, start = 0, stop = regexpr(pattern = "\\|", f) - 1))
@@ -306,34 +517,21 @@ find_formula.ivreg <- function(x, ...) {
       NULL
     }
   )
+  .find_formula_return(f)
 }
 
 
-
 #' @export
-find_formula.iv_robust <- function(x, ...) {
-  tryCatch(
-    {
-      f <- .safe_deparse(stats::formula(x))
-      cond <- .trim(gsub("(.*)\\+(\\s)*\\((.*)\\)", "\\1", f))
-      instr <- .trim(gsub("(.*)\\((.*)\\)", "\\2", f))
+find_formula.iv_robust <- find_formula.ivreg
 
-      list(
-        conditional = stats::as.formula(cond),
-        instruments = stats::as.formula(paste0("~", instr))
-      )
-    },
-    error = function(x) {
-      NULL
-    }
-  )
-}
 
+#' @export
+find_formula.ivFixed <- find_formula.ivreg
 
 
 #' @export
 find_formula.plm <- function(x, ...) {
-  tryCatch(
+  f <- tryCatch(
     {
       f <- .safe_deparse(stats::formula(x))
       bar_pos <- regexpr(pattern = "\\|", f)
@@ -362,6 +560,7 @@ find_formula.plm <- function(x, ...) {
       NULL
     }
   )
+  .find_formula_return(f)
 }
 
 
@@ -391,12 +590,13 @@ find_formula.felm <- function(x, ...) {
     f.clus <- NULL
   }
 
-  .compact_list(list(
+  f <- .compact_list(list(
     conditional = stats::as.formula(f.cond),
     random = stats::as.formula(f.rand),
     instruments = stats::as.formula(f.instr),
     cluster = stats::as.formula(f.clus)
   ))
+  .find_formula_return(f)
 }
 
 
@@ -420,11 +620,12 @@ find_formula.feglm <- function(x, ...) {
     f.clus <- NULL
   }
 
-  .compact_list(list(
+  f <- .compact_list(list(
     conditional = stats::as.formula(f.cond),
     instruments = stats::as.formula(f.instr),
     cluster = stats::as.formula(f.clus)
   ))
+  .find_formula_return(f)
 }
 
 
@@ -445,10 +646,11 @@ find_formula.fixest <- function(x, ...) {
     }
   }
 
-  .compact_list(list(
+  f <- .compact_list(list(
     conditional = stats::as.formula(f.cond),
     cluster = stats::as.formula(f.clus)
   ))
+  .find_formula_return(f)
 }
 
 
@@ -471,15 +673,46 @@ find_formula.feis <- function(x, ...) {
     f.slopes <- NULL
   }
 
-  .compact_list(list(
+  f <- .compact_list(list(
     conditional = stats::as.formula(f.cond),
     slopes = stats::as.formula(f.slopes),
     random = stats::as.formula(paste0("~", id))
   ))
+  .find_formula_return(f)
 }
 
 
 
+#' @export
+find_formula.bife <- function(x, ...) {
+  f <- .safe_deparse(stats::formula(x))
+  f_parts <- unlist(strsplit(f, "|", fixed = TRUE))
+
+  f.cond <- .trim(f_parts[1])
+
+  if (length(f_parts) > 1) {
+    f.rand <- paste0("~", .trim(f_parts[2]))
+  } else {
+    f.rand <- NULL
+  }
+
+  f <- .compact_list(list(
+    conditional = stats::as.formula(f.cond),
+    random = stats::as.formula(f.rand)
+  ))
+  .find_formula_return(f)
+}
+
+
+
+#' @export
+find_formula.ivprobit <- function(x, ...) {
+  NULL
+}
+
+
+
+#' @importFrom stats formula as.formula
 #' @export
 find_formula.wbm <- function(x, ...) {
   f <- .safe_deparse(stats::formula(x))
@@ -498,12 +731,9 @@ find_formula.wbm <- function(x, ...) {
     f_parts[3] <- .trim(f_parts[3])
     if (grepl("\\((.+)\\|(.+)\\)", f_parts[3])) {
       # we have multiple random effects, which we can better extract
-      # via "lme4::findbars()"
+      # via ".findbars()"
       if (length(gregexpr("\\|", f_parts[3])[[1]]) > 1) {
-        if (!requireNamespace("lme4", quietly = TRUE)) {
-          stop("To use this function, please install package 'lme4'.")
-        }
-        f.rand <- lme4::findbars(stats::as.formula(paste("~", f_parts[3])))
+        f.rand <- .findbars(stats::as.formula(paste("~", f_parts[3])))
       } else {
         f.rand <- gsub("(\\(|\\))", "", f_parts[3])
         f.rand <- stats::as.formula(paste0("~", .trim(f.rand)))
@@ -521,12 +751,13 @@ find_formula.wbm <- function(x, ...) {
     f.clint <- NULL
   }
 
-  .compact_list(list(
+  f <- .compact_list(list(
     conditional = stats::as.formula(f.cond),
     instruments = stats::as.formula(f.instr),
     interactions = stats::as.formula(f.clint),
     random = f.rand
   ))
+  .find_formula_return(f)
 }
 
 #' @export
@@ -535,17 +766,18 @@ find_formula.wbgee <- find_formula.wbm
 
 #' @export
 find_formula.glimML <- function(x, ...) {
-  .compact_list(list(
+  f <- .compact_list(list(
     conditional = x@formula,
     random = x@random
   ))
+  .find_formula_return(f)
 }
 
 
 
 #' @export
 find_formula.tobit <- function(x, ...) {
-  tryCatch(
+  f <- tryCatch(
     {
       list(conditional = parse(text = .safe_deparse(x$call))[[1]]$formula)
     },
@@ -553,6 +785,7 @@ find_formula.tobit <- function(x, ...) {
       NULL
     }
   )
+  .find_formula_return(f)
 }
 
 
@@ -574,6 +807,13 @@ find_formula.zeroinfl <- find_formula.hurdle
 find_formula.zerotrunc <- find_formula.hurdle
 
 
+#' @export
+find_formula.zcpglm <- function(x, ...) {
+  .zeroinf_formula(x, separator = "\\|\\|")
+}
+
+
+
 
 
 
@@ -586,21 +826,23 @@ find_formula.zerotrunc <- find_formula.hurdle
 #' @importFrom stats as.formula
 #' @export
 find_formula.clmm2 <- function(x, ...) {
-  .compact_list(list(
+  f <- .compact_list(list(
     conditional = stats::as.formula(.safe_deparse(attr(x$location, "terms", exact = TRUE))),
     scale = stats::as.formula(.safe_deparse(attr(x$scale, "terms", exact = TRUE))),
     random = stats::as.formula(paste0("~", parse(text = .safe_deparse(x$call))[[1]]$random))
   ))
+  .find_formula_return(f)
 }
 
 
 #' @importFrom stats formula
 #' @export
 find_formula.clm2 <- function(x, ...) {
-  .compact_list(list(
+  f <- .compact_list(list(
     conditional = stats::formula(attr(x$location, "terms", exact = TRUE)),
     scale = stats::formula(attr(x$scale, "terms", exact = TRUE))
   ))
+  .find_formula_return(f)
 }
 
 
@@ -633,7 +875,7 @@ find_formula.DirichletRegModel <- function(x, ...) {
     if (length(out) == 2) names(out)[2] <- "precision"
   }
 
-  out
+  .find_formula_return(out)
 }
 
 
@@ -652,26 +894,22 @@ find_formula.DirichletRegModel <- function(x, ...) {
 
 #' @export
 find_formula.glmmTMB <- function(x, ...) {
-  if (!requireNamespace("lme4", quietly = TRUE)) {
-    stop("To use this function, please install package 'lme4'.")
-  }
-
   f.cond <- stats::formula(x)
   f.zi <- stats::formula(x, component = "zi")
   f.disp <- stats::formula(x, component = "disp")
 
   if (identical(.safe_deparse(f.zi), "~0") ||
-    identical(.safe_deparse(f.zi), "~1")) {
+      identical(.safe_deparse(f.zi), "~1")) {
     f.zi <- NULL
   }
 
   if (identical(.safe_deparse(f.disp), "~0") ||
-    identical(.safe_deparse(f.disp), "~1")) {
+      identical(.safe_deparse(f.disp), "~1")) {
     f.disp <- NULL
   }
 
 
-  f.random <- lapply(lme4::findbars(f.cond), function(.x) {
+  f.random <- lapply(.findbars(f.cond), function(.x) {
     f <- .safe_deparse(.x)
     stats::as.formula(paste0("~", f))
   })
@@ -680,7 +918,7 @@ find_formula.glmmTMB <- function(x, ...) {
     f.random <- f.random[[1]]
   }
 
-  f.zirandom <- lapply(lme4::findbars(f.zi), function(.x) {
+  f.zirandom <- lapply(.findbars(f.zi), function(.x) {
     f <- .safe_deparse(.x)
     if (f == "NULL") {
       return(NULL)
@@ -696,24 +934,21 @@ find_formula.glmmTMB <- function(x, ...) {
   f.cond <- stats::as.formula(.get_fixed_effects(f.cond))
   if (!is.null(f.zi)) f.zi <- stats::as.formula(.get_fixed_effects(f.zi))
 
-  .compact_list(list(
+  f <- .compact_list(list(
     conditional = f.cond,
     random = f.random,
     zero_inflated = f.zi,
     zero_inflated_random = f.zirandom,
     dispersion = f.disp
   ))
+  .find_formula_return(f)
 }
 
 
 
 #' @export
 find_formula.nlmerMod <- function(x, ...) {
-  if (!requireNamespace("lme4", quietly = TRUE)) {
-    stop("To use this function, please install package 'lme4'.")
-  }
-
-  f.random <- lapply(lme4::findbars(stats::formula(x)), function(.x) {
+  f.random <- lapply(.findbars(stats::formula(x)), function(.x) {
     f <- .safe_deparse(.x)
     stats::as.formula(paste0("~", f))
   })
@@ -722,26 +957,23 @@ find_formula.nlmerMod <- function(x, ...) {
     f.random <- f.random[[1]]
   }
 
-  f.cond <- lme4::nobars(stats::as.formula(gsub("(.*)(~)(.*)~(.*)", "\\1\\2\\4", .safe_deparse(stats::formula(x)))))
+  f.cond <- .nobars(stats::as.formula(gsub("(.*)(~)(.*)~(.*)", "\\1\\2\\4", .safe_deparse(stats::formula(x)))))
   f.nonlin <- stats::as.formula(paste0("~", .trim(gsub("(.*)~(.*)~(.*)", "\\2", .safe_deparse(stats::formula(x))))))
 
-  .compact_list(list(
+  f <- .compact_list(list(
     conditional = f.cond,
     nonlinear = f.nonlin,
     random = f.random
   ))
+  .find_formula_return(f)
 }
 
 
 
 #' @export
 find_formula.merMod <- function(x, ...) {
-  if (!requireNamespace("lme4", quietly = TRUE)) {
-    stop("To use this function, please install package 'lme4'.")
-  }
-
   f.cond <- stats::formula(x)
-  f.random <- lapply(lme4::findbars(f.cond), function(.x) {
+  f.random <- lapply(.findbars(f.cond), function(.x) {
     f <- .safe_deparse(.x)
     stats::as.formula(paste0("~", f))
   })
@@ -751,7 +983,8 @@ find_formula.merMod <- function(x, ...) {
   }
 
   f.cond <- stats::as.formula(.get_fixed_effects(f.cond))
-  .compact_list(list(conditional = f.cond, random = f.random))
+  f <- .compact_list(list(conditional = f.cond, random = f.random))
+  .find_formula_return(f)
 }
 
 #' @export
@@ -775,6 +1008,36 @@ find_formula.cgamm <- find_formula.merMod
 #' @export
 find_formula.coxme <- find_formula.merMod
 
+#' @export
+find_formula.HLfit <- find_formula.merMod
+
+#' @export
+find_formula.merModList <- function(x, ...) {
+  find_formula(x[[1]], ...)
+}
+
+
+
+#' @export
+find_formula.sem <- function(x, ...) {
+  if (!.is_semLme(x)) {
+    return(NULL)
+  }
+
+  f.cond <- x$formula
+  f.random <- lapply(.findbars(f.cond), function(.x) {
+    f <- .safe_deparse(.x)
+    stats::as.formula(paste0("~", f))
+  })
+
+  if (length(f.random) == 1) {
+    f.random <- f.random[[1]]
+  }
+
+  f.cond <- stats::as.formula(.get_fixed_effects(f.cond))
+  f <- .compact_list(list(conditional = f.cond, random = f.random))
+  .find_formula_return(f)
+}
 
 
 #' @export
@@ -782,13 +1045,34 @@ find_formula.lme <- function(x, ...) {
   fm <- eval(x$call$fixed)
   fmr <- eval(x$call$random)
   ## TODO this is an intermediate fix to return the correlation variables from lme-objects
-  fc <- parse(text = .safe_deparse(x$call$correlation))[[1]]$form
+  fcorr <- x$call$correlation
+  if (!is.null(fcorr)) {
+    fc <- parse(text = .safe_deparse(x$call$correlation))[[1]]$form
+  } else {
+    fc <- NULL
+  }
 
-  .compact_list(list(
+  f <- .compact_list(list(
     conditional = fm,
     random = fmr,
     correlation = stats::as.formula(fc)
   ))
+  .find_formula_return(f)
+}
+
+
+
+#' @export
+find_formula.lqmm <- function(x, ...) {
+  fm <- eval(x$call$fixed)
+  fmr <- .safe_deparse(x$call$random)
+  fmg <- .safe_deparse(x$call$group)
+
+  f <- .compact_list(list(
+    conditional = fm,
+    random = stats::as.formula(paste0(fmr, "|", fmg))
+  ))
+  .find_formula_return(f)
 }
 
 
@@ -809,10 +1093,11 @@ find_formula.mixor <- function(x, ...) {
 
   fmr <- stats::as.formula(paste("~", fmr))
 
-  .compact_list(list(
+  f <- .compact_list(list(
     conditional = fm,
     random = fmr
   ))
+  .find_formula_return(f)
 }
 
 
@@ -824,12 +1109,13 @@ find_formula.MixMod <- function(x, ...) {
   f.random <- stats::formula(x, type = "random")
   f.zirandom <- stats::formula(x, type = "zi_random")
 
-  .compact_list(list(
+  f <- .compact_list(list(
     conditional = f.cond,
     random = f.random,
     zero_inflated = f.zi,
     zero_inflated_random = f.zirandom
   ))
+  .find_formula_return(f)
 }
 
 
@@ -839,17 +1125,18 @@ find_formula.BBmm <- function(x, ...) {
   f.cond <- parse(text = .safe_deparse(x$call))[[1]]$fixed.formula
   f.rand <- parse(text = .safe_deparse(x$call))[[1]]$random.formula
 
-  .compact_list(list(
+  f <- .compact_list(list(
     conditional = stats::as.formula(f.cond),
     random = stats::as.formula(f.rand)
   ))
+  .find_formula_return(f)
 }
 
 
 
 #' @export
 find_formula.mmclogit <- function(x, ...) {
-  tryCatch(
+  f <- tryCatch(
     {
       list(
         conditional = stats::formula(x),
@@ -860,9 +1147,28 @@ find_formula.mmclogit <- function(x, ...) {
       NULL
     }
   )
+  .find_formula_return(f)
 }
 
 
+#' @export
+find_formula.glmm <- function(x, ...) {
+  f.cond <- stats::as.formula(x$fixedcall)
+  f.random <- lapply(x$randcall, function(.x) {
+    av <- all.vars(.x)
+    stats::as.formula(paste0("~1|", av[length(av)]))
+  })
+
+  if (length(f.random) == 1) {
+    f.random <- f.random[[1]]
+  }
+
+  f <- .compact_list(list(conditional = f.cond, random = f.random))
+  .find_formula_return(f)
+}
+
+
+
 
 
 
@@ -870,25 +1176,54 @@ find_formula.mmclogit <- function(x, ...) {
 # Bayesian models --------------------------------
 
 
+#' @export
+find_formula.BGGM <- function(x, ...) {
+  list(conditional = x$formula)
+}
+
+
+
+#' @export
+find_formula.mcmc.list <- function(x, ...) {
+  NULL
+}
+
+
+
 #' @export
 find_formula.stanreg <- function(x, ...) {
-  if (!requireNamespace("lme4", quietly = TRUE)) {
-    stop("To use this function, please install package 'lme4'.")
+  if (inherits(x, "nlmerMod")) {
+    find_formula.nlmerMod(x, ...)
+  } else {
+    f.cond <- stats::formula(x)
+    # special handling for stan_gamm4
+    if (inherits(x, "gamm4")) {
+      f.random <- tryCatch(
+        {
+          lapply(.findbars(stats::formula(x$glmod)), function(.x) {
+            f <- .safe_deparse(.x)
+            stats::as.formula(paste0("~", f))
+          })
+        },
+        error = function(e) {
+          NULL
+        }
+      )
+    } else {
+      f.random <- lapply(.findbars(f.cond), function(.x) {
+        f <- .safe_deparse(.x)
+        stats::as.formula(paste0("~", f))
+      })
+    }
+
+    if (length(f.random) == 1) {
+      f.random <- f.random[[1]]
+    }
+
+    f.cond <- stats::as.formula(.get_fixed_effects(f.cond))
+    f <- .compact_list(list(conditional = f.cond, random = f.random))
+    .find_formula_return(f)
   }
-
-  f.cond <- stats::formula(x)
-  f.random <- lapply(lme4::findbars(f.cond), function(.x) {
-    f <- .safe_deparse(.x)
-    stats::as.formula(paste0("~", f))
-  })
-
-  if (length(f.random) == 1) {
-    f.random <- f.random[[1]]
-  }
-
-  f.cond <- stats::as.formula(.get_fixed_effects(f.cond))
-
-  .compact_list(list(conditional = f.cond, random = f.random))
 }
 
 
@@ -900,10 +1235,11 @@ find_formula.brmsfit <- function(x, ...) {
   if (.obj_has_name(f, "forms")) {
     mv_formula <- lapply(f$forms, .get_brms_formula)
     attr(mv_formula, "is_mv") <- "1"
-    mv_formula
+    f <- mv_formula
   } else {
-    .get_brms_formula(f)
+    f <- .get_brms_formula(f)
   }
+  .find_formula_return(f)
 }
 
 
@@ -913,7 +1249,7 @@ find_formula.stanmvreg <- function(x, ...) {
   f <- stats::formula(x)
   mv_formula <- lapply(f, .get_stanmv_formula)
   attr(mv_formula, "is_mv") <- "1"
-  mv_formula
+  .find_formula_return(mv_formula)
 }
 
 
@@ -923,7 +1259,8 @@ find_formula.MCMCglmm <- function(x, ...) {
   fm <- x$Fixed$formula
   fmr <- x$Random$formula
 
-  .compact_list(list(conditional = fm, random = fmr))
+  f <- .compact_list(list(conditional = fm, random = fmr))
+  .find_formula_return(f)
 }
 
 
@@ -940,19 +1277,34 @@ find_formula.BFBayesFactor <- function(x, ...) {
       f.random <- stats::as.formula(paste0("~", frand))
       fcond <- sub(frand, "", fcond, fixed = TRUE)
       fcond <- gsub("(.*)\\+$", "\\1", .trim(fcond))
+      # random effects only?
+      if (grepl("~$", fcond)) {
+        fcond <- paste(fcond, "1")
+      }
       f.cond <- stats::as.formula(.trim(fcond))
     } else {
       f.random <- NULL
       f.cond <- stats::as.formula(fcond)
     }
+  } else if (.classify_BFBayesFactor(x) %in% c("ttest1", "ttest2")) {
+    f.cond <- tryCatch(
+      {
+        stats::as.formula(x@numerator[[1]]@identifier$formula)
+      },
+      error = function(e) {
+        NULL
+      }
+    )
+    f.random <- NULL
   } else {
     return(NULL)
   }
 
-  .compact_list(list(
+  f <- .compact_list(list(
     conditional = f.cond,
     random = f.random
   ))
+  .find_formula_return(f)
 }
 
 
@@ -967,12 +1319,8 @@ find_formula.BFBayesFactor <- function(x, ...) {
 
 
 .get_brms_formula <- function(f) {
-  if (!requireNamespace("lme4", quietly = TRUE)) {
-    stop("To use this function, please install package 'lme4'.")
-  }
-
   f.cond <- f$formula
-  f.random <- lapply(lme4::findbars(f.cond), function(.x) {
+  f.random <- lapply(.findbars(f.cond), function(.x) {
     fm <- .safe_deparse(.x)
     stats::as.formula(paste0("~", fm))
   })
@@ -987,7 +1335,7 @@ find_formula.BFBayesFactor <- function(x, ...) {
   f.zirandom <- NULL
 
   if (!.is_empty_object(f.zi)) {
-    f.zirandom <- lapply(lme4::findbars(f.zi), function(.x) {
+    f.zirandom <- lapply(.findbars(f.zi), function(.x) {
       f <- .safe_deparse(.x)
       stats::as.formula(paste0("~", f))
     })
@@ -1012,12 +1360,8 @@ find_formula.BFBayesFactor <- function(x, ...) {
 
 
 .get_stanmv_formula <- function(f) {
-  if (!requireNamespace("lme4", quietly = TRUE)) {
-    stop("To use this function, please install package 'lme4'.")
-  }
-
   f.cond <- f
-  f.random <- lapply(lme4::findbars(f.cond), function(.x) {
+  f.random <- lapply(.findbars(f.cond), function(.x) {
     fm <- .safe_deparse(.x)
     stats::as.formula(paste0("~", fm))
   })
@@ -1038,7 +1382,7 @@ find_formula.BFBayesFactor <- function(x, ...) {
 
 # Find formula for zero-inflated regressions, where
 # zero-inflated part is separated by | from count part
-.zeroinf_formula <- function(x) {
+.zeroinf_formula <- function(x, separator = "\\|") {
   f <- tryCatch(
     {
       stats::formula(x)
@@ -1052,7 +1396,7 @@ find_formula.BFBayesFactor <- function(x, ...) {
     return(NULL)
   }
 
-  f <- .trim(unlist(strsplit(.safe_deparse(f), "\\|")))
+  f <- .trim(unlist(strsplit(.safe_deparse(f), separator)))
 
   c.form <- stats::as.formula(f[1])
   if (length(f) == 2) {
@@ -1082,7 +1426,8 @@ find_formula.BFBayesFactor <- function(x, ...) {
   )
 
 
-  .compact_list(list(conditional = c.form, zero_inflated = zi.form))
+  f <- .compact_list(list(conditional = c.form, zero_inflated = zi.form))
+  .find_formula_return(f)
 }
 
 
@@ -1105,3 +1450,71 @@ find_formula.BFBayesFactor <- function(x, ...) {
     }
   )
 }
+
+
+.fix_gamm_random_effect <- function(x) {
+  g_in_terms <- length(x) > 1 && x[length(x)] == "g"
+  xr_in_terms <- length(x) > 1 && x[length(x)] == "Xr"
+  x <- x[!(grepl("(Xr\\.\\d|g\\.\\d)", x) | x %in% c("Xr", "g"))]
+  # exceptions, if random effect is named g
+  if (!length(x) && isTRUE(g_in_terms)) {
+    x <- "g"
+  }
+  if (!length(x) && isTRUE(xr_in_terms)) {
+    x <- "Xr"
+  }
+  x
+}
+
+
+.fix_gamm4_random_effect <- function(f) {
+  if (inherits(f, "formula")) {
+    f <- list(f)
+  }
+  len <- length(f)
+  keep <- sapply(f, function(i) {
+    i <- gsub("(~1| | \\|)", "", deparse(i))
+    !any(grepl("(Xr\\.\\d|g\\.\\d)", i) | i %in% c("Xr", "g"))
+  })
+  f <- .compact_list(f[keep])
+  # exceptions, if random effect is named Xr
+  if (!length(f) && len > 1) {
+    f <- list(stats::as.formula("~1 | Xr"))
+  }
+  f
+}
+
+
+
+# Helpers and Methods -----------------------------------------------------
+
+
+.find_formula_return <- function(f) {
+  if (is.null(f)) return(NULL)
+  class(f) <- c("insight_formula", class(f))
+  f
+}
+
+
+
+#' @export
+format.insight_formula <- function(x, what = c("conditional", "random"), ...) {
+  # The purpose of this function is to flatten the formula
+
+  # Start by first part (conditional by default)
+  ft <- format(x[[1]])
+
+  # Wrap random in brackets
+  if ("random" %in% names(x)) {
+    x[["random"]] <- paste0("(", format(x[["random"]]), ")")
+  }
+
+  # Add all the components
+  for (part in what[-1]) {
+    if(part %in% names(x)){
+      ft <- paste0(ft, " + ", format(x[[part]]))
+    }
+  }
+
+  ft
+}
