@@ -5,7 +5,8 @@
 #'   model and returns it as data frame.
 #'
 #' @param effects Should model data for fixed effects, random effects
-#'    or both be returned? Only applies to mixed models.
+#'   or both be returned? Only applies to mixed models.
+#' @param verbose Toggle messages and warnings.
 #'
 #' @inheritParams find_predictors
 #' @inheritParams find_formula
@@ -15,7 +16,9 @@
 #' @note Unlike \code{model.frame()}, which may contain transformed variables
 #'   (e.g. if \code{poly()} or \code{scale()} was used inside the formula to
 #'   specify the model), \code{get_data()} aims at returning the "original",
-#'   untransformed data.
+#'   untransformed data (if possible). Consequently, column names are changed
+#'   accordingly, i.e. \code{"log(x)"} will become \code{"x"} etc. for all data
+#'   columns with transformed values.
 #'
 #' @examples
 #' data(cbpp, package = "lme4")
@@ -33,7 +36,7 @@ get_data <- function(x, ...) {
 
 
 #' @export
-get_data.default <- function(x, ...) {
+get_data.default <- function(x, verbose = TRUE, ...) {
   if (inherits(x, "list") && .obj_has_name(x, "gam")) {
     x <- x$gam
     class(x) <- c(class(x), c("glm", "lm"))
@@ -63,7 +66,7 @@ get_data.default <- function(x, ...) {
     )
   }
 
-  .prepare_get_data(x, mf)
+  .prepare_get_data(x, mf, verbose = verbose)
 }
 
 
@@ -73,6 +76,20 @@ get_data.data.frame <- function(x, ...) {
 }
 
 
+#' @export
+get_data.summary.lm <- function(x, ...) {
+  mf <- tryCatch(
+    {
+      .get_data_from_env(x)[, all.vars(x$terms), drop = FALSE]
+    },
+    error = function(x) {
+      NULL
+    }
+  )
+  .prepare_get_data(x, mf)
+}
+
+
 
 
 
@@ -82,7 +99,7 @@ get_data.data.frame <- function(x, ...) {
 
 #' @rdname get_data
 #' @export
-get_data.gee <- function(x, effects = c("all", "fixed", "random"), ...) {
+get_data.gee <- function(x, effects = c("all", "fixed", "random"), verbose = TRUE, ...) {
   effects <- match.arg(effects)
   mf <- tryCatch(
     {
@@ -99,12 +116,14 @@ get_data.gee <- function(x, effects = c("all", "fixed", "random"), ...) {
     }
   )
 
-  .prepare_get_data(x, stats::na.omit(mf))
+  .prepare_get_data(x, stats::na.omit(mf), effects = effects, verbose = verbose)
 }
 
+
+
 #' @rdname get_data
 #' @export
-get_data.rqss <- function(x, component = c("all", "conditional", "smooth_terms"), ...) {
+get_data.rqss <- function(x, component = c("all", "conditional", "smooth_terms"), verbose = TRUE, ...) {
   component <- match.arg(component)
   mf <- tryCatch(
     {
@@ -116,7 +135,7 @@ get_data.rqss <- function(x, component = c("all", "conditional", "smooth_terms")
     }
   )
 
-  .prepare_get_data(x, stats::na.omit(mf))
+  .prepare_get_data(x, stats::na.omit(mf), verbose = verbose)
 }
 
 
@@ -124,7 +143,7 @@ get_data.rqss <- function(x, component = c("all", "conditional", "smooth_terms")
 
 
 #' @export
-get_data.gls <- function(x, ...) {
+get_data.gls <- function(x, verbose = TRUE, ...) {
   mf <- tryCatch(
     {
       dat <- .get_data_from_env(x)
@@ -136,7 +155,7 @@ get_data.gls <- function(x, ...) {
     }
   )
 
-  .prepare_get_data(x, stats::na.omit(mf))
+  .prepare_get_data(x, stats::na.omit(mf), verbose = verbose)
 }
 
 #' @export
@@ -151,6 +170,25 @@ get_data.complmrob <- get_data.gls
 #' @export
 get_data.nlrq <- get_data.gls
 
+#' @export
+get_data.robmixglm <- get_data.gls
+
+
+#' @export
+get_data.lqmm <- function(x, verbose = TRUE, ...) {
+  mf <- tryCatch(
+    {
+      x$mfArgs$data
+    },
+    error = function(x) {
+      NULL
+    }
+  )
+
+  .prepare_get_data(x, stats::na.omit(mf), verbose = verbose)
+}
+
+
 
 
 
@@ -164,16 +202,48 @@ get_data.nlrq <- get_data.gls
 
 #' @rdname get_data
 #' @export
-get_data.hurdle <- function(x, component = c("all", "conditional", "zi", "zero_inflated", "dispersion"), ...) {
+get_data.hurdle <- function(x, component = c("all", "conditional", "zi", "zero_inflated", "dispersion"), verbose = TRUE, ...) {
   component <- match.arg(component)
-  .return_zeroinf_data(x, component)
+  .return_zeroinf_data(x, component, verbose = verbose)
 }
 
 #' @export
 get_data.zeroinfl <- get_data.hurdle
 
 #' @export
-get_data.zerotrunc <- get_data.default
+get_data.zerotrunc <- get_data.hurdle
+
+
+#' @rdname get_data
+#' @export
+get_data.zcpglm <- function(x, component = c("all", "conditional", "zi", "zero_inflated"), verbose = TRUE, ...) {
+  component <- match.arg(component)
+
+  mf <- stats::model.frame(x)
+  mf_zero <- mf$zero
+  mf_tweedie <- mf$tweedie
+
+  # zcpglm saves variables twice, once in the model frame for zero-inflated
+  # model and once for the tweedie-model. we now need to remove duplicates
+  cn <- setdiff(colnames(mf$zero), colnames(mf$tweedie))
+
+  if (length(cn)) {
+    mf_zero <- mf_zero[cn]
+  } else {
+    mf_zero <- NULL
+  }
+
+  mf <- switch(
+    component,
+    "all" = do.call(cbind, .compact_list(list(mf_tweedie, mf_zero))),
+    "conditional" = mf_tweedie,
+    "zi" = ,
+    "zero_inflated" = mf_zero
+  )
+
+  .prepare_get_data(x, stats::na.omit(mf), verbose = verbose)
+}
+
 
 
 
@@ -187,7 +257,7 @@ get_data.zerotrunc <- get_data.default
 
 #' @rdname get_data
 #' @export
-get_data.glmmTMB <- function(x, effects = c("all", "fixed", "random"), component = c("all", "conditional", "zi", "zero_inflated", "dispersion"), ...) {
+get_data.glmmTMB <- function(x, effects = c("all", "fixed", "random"), component = c("all", "conditional", "zi", "zero_inflated", "dispersion"), verbose = TRUE, ...) {
   effects <- match.arg(effects)
   component <- match.arg(component)
 
@@ -202,21 +272,21 @@ get_data.glmmTMB <- function(x, effects = c("all", "fixed", "random"), component
     }
   )
 
-  mf <- .prepare_get_data(x, mf)
+  mf <- .prepare_get_data(x, mf, effects, verbose = verbose)
 
   # add variables from other model components
   mf <- .add_zeroinf_data(x, mf, model.terms$dispersion)
   mf <- .add_zeroinf_data(x, mf, model.terms$zero_inflated)
   mf <- .add_zeroinf_data(x, mf, model.terms$zero_inflated_random)
 
-  .return_data(x, mf, effects, component, model.terms)
+  .return_data(x, mf, effects, component, model.terms, verbose = verbose)
 }
 
 
 
 #' @rdname get_data
 #' @export
-get_data.merMod <- function(x, effects = c("all", "fixed", "random"), ...) {
+get_data.merMod <- function(x, effects = c("all", "fixed", "random"), verbose = TRUE, ...) {
   effects <- match.arg(effects)
 
   mf <- tryCatch(
@@ -233,13 +303,19 @@ get_data.merMod <- function(x, effects = c("all", "fixed", "random"), ...) {
     }
   )
 
-  .prepare_get_data(x, mf, effects)
+  .prepare_get_data(x, mf, effects, verbose = verbose)
+}
+
+#' @export
+get_data.merModList <- function(x, effects = c("all", "fixed", "random"), ...) {
+  warning("Can't access data for 'merModList' objects.", call. = FALSE)
+  return(NULL)
 }
 
 
 
 #' @export
-get_data.MANOVA <- function(x, effects = c("all", "fixed", "random"), ...) {
+get_data.MANOVA <- function(x, effects = c("all", "fixed", "random"), verbose = TRUE, ...) {
   effects <- match.arg(effects)
 
   mf <- tryCatch(
@@ -256,7 +332,7 @@ get_data.MANOVA <- function(x, effects = c("all", "fixed", "random"), ...) {
     }
   )
 
-  .prepare_get_data(x, mf, effects)
+  .prepare_get_data(x, mf, effects, verbose = verbose)
 }
 
 #' @export
@@ -265,7 +341,7 @@ get_data.RM <- get_data.MANOVA
 
 
 #' @export
-get_data.cpglmm <- function(x, effects = c("all", "fixed", "random"), ...) {
+get_data.cpglmm <- function(x, effects = c("all", "fixed", "random"), verbose = TRUE, ...) {
   effects <- match.arg(effects)
   dat <- stats::model.frame(x)
 
@@ -283,13 +359,37 @@ get_data.cpglmm <- function(x, effects = c("all", "fixed", "random"), ...) {
     }
   )
 
-  .prepare_get_data(x, mf, effects)
+  .prepare_get_data(x, mf, effects, verbose = verbose)
+}
+
+#' @export
+get_data.HLfit <- get_data.cpglmm
+
+
+#' @export
+get_data.glmm <- function(x, effects = c("all", "fixed", "random"), verbose = TRUE, ...) {
+  effects <- match.arg(effects)
+  dat <- get_data.default(x, verbose = verbose)
+
+  mf <- tryCatch(
+    {
+      switch(
+        effects,
+        fixed = dat[, find_predictors(x, effects = "fixed", flatten = TRUE), drop = FALSE],
+        all = dat,
+        random = dat[, find_random(x, split_nested = TRUE, flatten = TRUE), drop = FALSE]
+      )
+    },
+    error = function(x) {
+      NULL
+    }
+  )
 }
 
 
 
 #' @export
-get_data.mixor <- function(x, effects = c("all", "fixed", "random"), ...) {
+get_data.mixor <- function(x, effects = c("all", "fixed", "random"), verbose = TRUE, ...) {
   effects <- match.arg(effects)
 
   mf <- tryCatch(
@@ -309,7 +409,7 @@ get_data.mixor <- function(x, effects = c("all", "fixed", "random"), ...) {
   fix_cn <- which(colnames(mf) %in% c("x.id", "x$id"))
   colnames(mf)[fix_cn] <- deparse(x$call$id)
 
-  .prepare_get_data(x, mf, effects)
+  .prepare_get_data(x, mf, effects, verbose = verbose)
 }
 
 
@@ -361,6 +461,36 @@ get_data.mixed <- function(x, effects = c("all", "fixed", "random"), ...) {
 
 
 
+#' @export
+get_data.afex_aov <- function(x, ...) {
+  .get_data_from_modelframe(x, x$data$long, "all")
+}
+
+
+
+#' @export
+get_data.sem <- function(x, effects = c("all", "fixed", "random"), ...) {
+  effects <- match.arg(effects)
+  mf <- tryCatch(
+    {
+      dat <- .get_data_from_env(x)
+      switch(
+        effects,
+        all = dat[, find_variables(x, flatten = TRUE), drop = FALSE],
+        fixed = dat[, find_variables(x, effects = "fixed", flatten = TRUE), drop = FALSE],
+        random = dat[, find_random(x, flatten = TRUE), drop = FALSE]
+      )
+    },
+    error = function(x) {
+      NULL
+    }
+  )
+
+  .prepare_get_data(x, stats::na.omit(mf), effects)
+}
+
+
+
 #' @rdname get_data
 #' @export
 get_data.lme <- function(x, effects = c("all", "fixed", "random"), ...) {
@@ -379,33 +509,42 @@ get_data.lme <- function(x, effects = c("all", "fixed", "random"), ...) {
 
 
 
+#' @importFrom stats model.frame
 #' @rdname get_data
 #' @export
-get_data.MixMod <- function(x, effects = c("all", "fixed", "random"), component = c("all", "conditional", "zi", "zero_inflated", "dispersion"), ...) {
+get_data.MixMod <- function(x, effects = c("all", "fixed", "random"), component = c("all", "conditional", "zi", "zero_inflated", "dispersion"), verbose = TRUE, ...) {
   effects <- match.arg(effects)
   component <- match.arg(component)
 
   tryCatch(
     {
-      fitfram <- x$model_frames$mfX
-      if (!.is_empty_object(x$model_frames$mfZ)) {
-        fitfram <- .merge_dataframes(x$model_frames$mfZ, fitfram, replace = TRUE)
+      fitfram <- stats::model.frame(x, type = "fixed")
+      fitfram_re <- stats::model.frame(x, type = "random")
+      fitfram_zi <- stats::model.frame(x, type = "zi_fixed")
+      fitfram_zi_re <- stats::model.frame(x, type = "zi_random")
+
+      if (!.is_empty_object(fitfram_re)) {
+        for (i in 1:length(fitfram_re)) {
+          fitfram <- .merge_dataframes(fitfram_re[[i]], fitfram, replace = TRUE)
+        }
       }
-      if (!.is_empty_object(x$model_frames$mfX_zi)) {
-        fitfram <- .merge_dataframes(x$model_frames$mfX_zi, fitfram, replace = TRUE)
+      if (!.is_empty_object(fitfram_zi)) {
+        fitfram <- .merge_dataframes(fitfram_zi, fitfram, replace = TRUE)
       }
-      if (!.is_empty_object(x$model_frames$mfZ_zi)) {
-        fitfram <- .merge_dataframes(x$model_frames$mfZ_zi, fitfram, replace = TRUE)
+      if (!.is_empty_object(fitfram_zi_re)) {
+        for (i in 1:length(fitfram_zi_re)) {
+          fitfram <- .merge_dataframes(fitfram_zi_re[[i]], fitfram, replace = TRUE)
+        }
       }
 
       fitfram$grp__id <- unlist(x$id)
       colnames(fitfram)[ncol(fitfram)] <- x$id_name[1]
 
       # test...
-      fitfram <- .prepare_get_data(x, fitfram)
+      fitfram <- .prepare_get_data(x, fitfram, effects)
 
       model.terms <- find_variables(x, effects = "all", component = "all", flatten = FALSE)
-      .return_data(x, mf = fitfram, effects, component, model.terms)
+      .return_data(x, mf = fitfram, effects, component, model.terms, verbose = verbose)
     },
     error = function(x) {
       NULL
@@ -433,7 +572,7 @@ get_data.BBmm <- function(x, effects = c("all", "fixed", "random"), ...) {
     }
   )
 
-  .prepare_get_data(x, stats::na.omit(mf))
+  .prepare_get_data(x, stats::na.omit(mf), effects)
 }
 
 
@@ -449,13 +588,14 @@ get_data.glimML <- function(x, effects = c("all", "fixed", "random"), ...) {
     random = dat[, find_random(x, flatten = TRUE), drop = FALSE]
   )
 
-  .prepare_get_data(x, stats::na.omit(mf))
+  .prepare_get_data(x, stats::na.omit(mf), effects)
 }
 
 
 
 
 
+
 # sem models -------------------------------------
 
 #' @export
@@ -620,7 +760,7 @@ get_data.wbm <- function(x, effects = c("all", "fixed", "random"), ...) {
   resp.col <- which(colnames(mf) == find_response(x))
   mf <- mf[, c(resp.col, (1:ncol(mf))[-resp.col])]
 
-  .prepare_get_data(x, stats::na.omit(mf))
+  .prepare_get_data(x, stats::na.omit(mf), effects)
 }
 
 
@@ -657,6 +797,28 @@ get_data.ivreg <- function(x, ...) {
 get_data.iv_robust <- get_data.ivreg
 
 
+#' @export
+get_data.ivprobit <- function(x, ...) {
+  .prepare_get_data(x, stats::na.omit(as.data.frame(x$mr1)))
+}
+
+
+#' @export
+get_data.bife <- function(x, effects = c("all", "fixed", "random"), ...) {
+  effects <- match.arg(effects)
+  mf <- as.data.frame(x$data)
+
+  if (effects == "random") {
+    return(stats::na.omit(mf[, unique(find_random(x, split_nested = TRUE, flatten = TRUE)), drop = FALSE]))
+  } else if (effects == "fixed") {
+    mf <- mf[, setdiff(colnames(mf), unique(find_random(x, split_nested = TRUE, flatten = TRUE))), drop = FALSE]
+  }
+
+  .prepare_get_data(x, stats::na.omit(mf), effects)
+}
+
+
+
 
 
 
@@ -668,7 +830,7 @@ get_data.iv_robust <- get_data.ivreg
 
 #' @rdname get_data
 #' @export
-get_data.brmsfit <- function(x, effects = c("all", "fixed", "random"), component = c("all", "conditional", "zi", "zero_inflated"), ...) {
+get_data.brmsfit <- function(x, effects = c("all", "fixed", "random"), component = c("all", "conditional", "zi", "zero_inflated"), verbose = TRUE, ...) {
   effects <- match.arg(effects)
   component <- match.arg(component)
 
@@ -687,7 +849,8 @@ get_data.brmsfit <- function(x, effects = c("all", "fixed", "random"), component
     effects,
     component,
     model.terms,
-    is_mv = is_multivariate(x)
+    is_mv = is_multivariate(x),
+    verbose = verbose
   )
 }
 
@@ -695,7 +858,7 @@ get_data.brmsfit <- function(x, effects = c("all", "fixed", "random"), component
 
 #' @rdname get_data
 #' @export
-get_data.stanreg <- function(x, effects = c("all", "fixed", "random"), ...) {
+get_data.stanreg <- function(x, effects = c("all", "fixed", "random"), verbose = TRUE, ...) {
   effects <- match.arg(effects)
 
   model.terms <- find_variables(x, effects = "all", component = "all", flatten = FALSE)
@@ -703,11 +866,12 @@ get_data.stanreg <- function(x, effects = c("all", "fixed", "random"), ...) {
 
   .return_data(
     x,
-    .prepare_get_data(x, mf, effects = effects),
+    .prepare_get_data(x, mf, effects = effects, verbose = verbose),
     effects,
     component = "all",
     model.terms,
-    is_mv = is_multivariate(x)
+    is_mv = is_multivariate(x),
+    verbose = verbose
   )
 }
 
@@ -737,7 +901,7 @@ get_data.MCMCglmm <- function(x, effects = c("all", "fixed", "random"), ...) {
         dat <- get(mf)
         switch(
           effects,
-          fixed = dat[, setdiff(colnames(dat), find_random(x, flatten = T)), drop = FALSE],
+          fixed = dat[, setdiff(colnames(dat), find_random(x, flatten = TRUE)), drop = FALSE],
           all = dat,
           random = dat[, find_random(x, flatten = TRUE), drop = FALSE]
         )
@@ -779,9 +943,141 @@ get_data.stanmvreg <- function(x, ...) {
 
 
 
+# mfx models ------------------------------------------------------
+
+#' @export
+get_data.betamfx <- function(x, ...) {
+  get_data(x$fit, ...)
+}
+
+#' @export
+get_data.betaor <- get_data.betamfx
+
+#' @export
+get_data.logitor <- get_data.betamfx
+
+#' @export
+get_data.poissonirr <- get_data.betamfx
+
+#' @export
+get_data.negbinirr <- get_data.betamfx
+
+#' @export
+get_data.logitmfx <- get_data.betamfx
+
+#' @export
+get_data.poissonmfx <- get_data.betamfx
+
+#' @export
+get_data.probitmfx <- get_data.betamfx
+
+#' @export
+get_data.negbinmfx <- get_data.betamfx
+
+
+
+
+
+
 # other models ------------------------------------------------------
 
 
+#' @export
+get_data.mediate <- function(x, ...) {
+  d1 <- get_data(x$model.m)
+  d2 <- get_data(x$model.y)
+  merge(d1, d2, sort = FALSE, all = TRUE)
+}
+
+
+#' @export
+get_data.mle2 <- function(x, ...) {
+  as.data.frame(do.call(cbind, x@data))
+}
+
+#' @export
+get_data.mle <- get_data.mle2
+
+
+
+#' @export
+get_data.glht <- function(x, ...) {
+  get_data(x$model, ...)
+}
+
+
+#' @export
+get_data.averaging <- function(x, ...) {
+  ml <- attributes(x)$modelList
+  if (is.null(ml)) {
+    warning("Can't retrieve data. Please use 'fit = TRUE' in 'model.avg()'.", call. = FALSE)
+    return(NULL)
+  }
+
+  mf <- tryCatch(
+    {
+      Reduce(function(x, y) merge(x, y, all = TRUE, sort = FALSE), lapply(ml, stats::model.frame))
+    },
+    error = function(x) {
+      NULL
+    }
+  )
+
+  if (is.null(mf)) {
+    mf <- tryCatch(
+      {
+        .get_data_from_env(x)[, find_variables(x, flatten = TRUE), drop = FALSE]
+      },
+      error = function(x) {
+        NULL
+      }
+    )
+  }
+
+  .prepare_get_data(x, mf)
+}
+
+
+#' @export
+get_data.Arima <- function(x, ...) {
+  # first try, parent frame
+  dat <- tryCatch(
+    {
+      eval(x$call$x, envir = parent.frame())
+    },
+    error = function(e) {
+      NULL
+    }
+  )
+
+  if (is.null(dat)) {
+    # second try, global env
+    dat <- tryCatch(
+      {
+        eval(x$call$x, envir = globalenv())
+      },
+      error = function(e) {
+        NULL
+      }
+    )
+  }
+
+  dat
+}
+
+
+#' @export
+get_data.BGGM <- function(x, ...) {
+  x$Y
+}
+
+
+#' @export
+get_data.mcmc.list <- function(x, ...) {
+  NULL
+}
+
+
 #' @export
 get_data.DirichletRegModel <- function(x, ...) {
   mf <- x$data
@@ -843,7 +1139,7 @@ get_data.LORgee <- function(x, effects = c("all", "fixed", "random"), ...) {
     }
   )
 
-  .prepare_get_data(x, stats::na.omit(mf))
+  .prepare_get_data(x, stats::na.omit(mf), effects = effects)
 }
 
 
@@ -949,6 +1245,22 @@ get_data.bracl <- function(x, ...) {
 
 
 
+#' @export
+get_data.mlogit <- function(x, ...) {
+  mf <- tryCatch(
+    {
+      as.data.frame(stats::model.frame(x))
+    },
+    error = function(x) {
+      NULL
+    }
+  )
+
+  .prepare_get_data(x, mf)
+}
+
+
+
 #' @export
 get_data.rma <- function(x, ...) {
   mf <- tryCatch(
@@ -962,3 +1274,72 @@ get_data.rma <- function(x, ...) {
 
   .prepare_get_data(x, stats::na.omit(mf))
 }
+
+
+#' @export
+get_data.metaplus <- get_data.rma
+
+
+#' @export
+get_data.meta_random <- function(x, ...) {
+  mf <- tryCatch(
+    {
+      x$data$data
+    },
+    error = function(x) {
+      NULL
+    }
+  )
+
+  .prepare_get_data(x, stats::na.omit(mf))
+}
+
+
+#' @export
+get_data.meta_bma <- function(x, ...) {
+  mf <- tryCatch(
+    {
+      x$meta$fixed$data$data
+    },
+    error = function(x) {
+      NULL
+    }
+  )
+
+  .prepare_get_data(x, stats::na.omit(mf))
+}
+
+
+#' @export
+get_data.meta_fixed <- get_data.meta_random
+
+
+#' @export
+get_data.ivFixed <- get_data.rma
+
+
+#' @export
+get_data.mipo <- function(x, ...) {
+  tryCatch(
+    {
+      models <- eval(x$call$object)
+      get_data(models$analyses[[1]], ...)
+    },
+    error = function(e) {
+      NULL
+    }
+  )
+}
+
+
+#' @export
+get_data.htest <- function(x, ...) {
+  out <- NULL
+  if (!is.null(x$data.name)) {
+    for (parent_level in 1:5) {
+      out <- .retrieve_htest_data(x, parent_level)
+      if (!is.null(out)) break
+    }
+  }
+  out
+}
