@@ -4,12 +4,12 @@
 
 
 # remove NULL elements from lists
-.compact_list <- function(x) x[!sapply(x, function(i) length(i) == 0 || is.null(i) || (!is.data.frame(i) && any(i == "NULL")) || (is.data.frame(i) && nrow(i) == 0))]
+.compact_list <- function(x) x[!sapply(x, function(i) length(i) == 0 || is.null(i) || (!is.data.frame(i) && any(i == "NULL", na.rm = TRUE)) || (is.data.frame(i) && nrow(i) == 0))]
 
 
 
 # remove empty string from character
-.compact_character <- function(x) x[!sapply(x, function(i) nchar(i) == 0 || is.null(i) || any(i == "NULL"))]
+.compact_character <- function(x) x[!sapply(x, function(i) nchar(i) == 0 || is.null(i) || any(i == "NULL", na.rm = TRUE))]
 
 
 
@@ -122,54 +122,11 @@
 
 
 # removes random effects from a formula that is in lmer-notation
-#' @importFrom stats terms drop.terms update
 .get_fixed_effects <- function(f) {
-  f_string <- .safe_deparse(f)
-
-  # for some weird brms-models, we also have a "|" in the response.
-  # in order to check for "|" only in the random effects, we have
-  # to remove the response here...
-
-  f_response <- .safe_deparse(f[[2]])
-  f_predictors <- sub(f_response, "", f_string, fixed = TRUE)
-
-  if (grepl("|", f_predictors, fixed = TRUE)) {
-    if (length(f) > 2) {
-      f_cond <- .safe_deparse(f[[3]])
-    } else {
-      f_cond <- .safe_deparse(f[[length(f)]])
-    }
-    # get first parentheses. for mixed models, might not be the random effects
-    # e.g.: (country + cohort + country * cohort) + age + (1 + age_c | mergeid)
-    first_parentheses <- gsub("^\\((.*?)\\)(.*)", "\\1", f_cond)
-    has_bar <- grepl("\\|", first_parentheses)
-    starts_with_parantheses <- grepl("^\\(", f_cond)
-    # intercept only model, w/o "1" in formula notation?
-    # e.g. "Reaction ~ (1 + Days | Subject)"
-    if (length(f) > 2 && starts_with_parantheses && has_bar) {
-      # check if we have any terms *after* random effects, e.g.
-      # social ~ (1|school) + open + extro + agree + school
-      if (.formula_empty_after_random_effect(f_predictors)) {
-        # here we fix intercept only models with random effects,
-        # like social ~ (1|school).
-        .trim(paste0(.safe_deparse(f[[2]]), " ~ 1"))
-      } else {
-        # here we fix models where random effects come first,
-        # like social ~ (1|school) + open + extro + agree + school
-        # the regex removes "(1|school) + "
-        .trim(gsub("\\((.*)\\)(\\s)*\\+(\\s)*", "", f_string))
-      }
-    } else if (!grepl("\\+(\\s)*\\((.*)\\)", f_string)) {
-      f_terms <- stats::terms(f)
-      pos_bar <- grep("|", labels(f_terms), fixed = TRUE)
-      no_bars <- stats::drop.terms(f_terms, pos_bar, keep.response = TRUE)
-      stats::update(f_terms, no_bars)
-    } else {
-      .trim(gsub("\\+(\\s)*\\((.*)\\)", "", f_string))
-    }
-  } else {
-    .trim(gsub("\\+(\\s)*\\((.*)\\)", "", f_string))
-  }
+  # remove random effects from RHS
+  fl <- length(f)
+  f[[fl]] <- .nobars(f[[fl]])
+  f
 }
 
 
@@ -187,21 +144,18 @@
 # extract random effects from formula
 .get_model_random <- function(f, split_nested = FALSE, model) {
   is_special <- inherits(
-      model,
-      c("MCMCglmm", "gee", "LORgee", "mixor", "clmm2", "felm", "feis", "bife",
-        "BFBayesFactor", "BBmm", "glimML", "MANOVA", "RM", "cglm")
+    model,
+    c(
+      "MCMCglmm", "gee", "LORgee", "mixor", "clmm2", "felm", "feis", "bife",
+      "BFBayesFactor", "BBmm", "glimML", "MANOVA", "RM", "cglm", "glmm"
     )
+  )
 
-  if (identical(.safe_deparse(f), "~0") ||
-    identical(.safe_deparse(f), "~1")) {
+  if (identical(.safe_deparse(f), "~0") || identical(.safe_deparse(f), "~1")) {
     return(NULL)
   }
 
-  if (!requireNamespace("lme4", quietly = TRUE)) {
-    stop("To use this function, please install package 'lme4'.")
-  }
-
-  re <- sapply(lme4::findbars(f), .safe_deparse)
+  re <- sapply(.findbars(f), .safe_deparse)
 
   if (is_special && .is_empty_object(re)) {
     re <- all.vars(f[[2L]])
@@ -281,14 +235,44 @@
 
 
 # to reduce redundant code, I extract this part which is used several
-# times accross this package
+# times across this package
 .get_elements <- function(effects, component) {
-  elements <- c("conditional", "conditional2", "conditional3", "precision", "nonlinear", "random", "zero_inflated", "zero_inflated_random", "dispersion", "instruments", "interactions", "simplex", "smooth_terms", "sigma", "nu", "tau", "correlation", "slopes", "cluster", "extra", "scale")
+
+  # all elements of a model
+  elements <- c("conditional", "conditional2", "conditional3", "precision",
+                "nonlinear", "random", "zero_inflated", "zero_inflated_random",
+                "dispersion", "instruments", "interactions", "simplex",
+                "smooth_terms", "sigma", "nu", "tau", "correlation", "slopes",
+                "cluster", "extra", "scale", "marginal", "alpha", "beta")
+
+  # auxiliary parameters
+  auxiliary_parameters <- c("sigma", "alpha", "beta", "dispersion", "precision", "nu", "tau", "shape", "phi", "ndt", "hu", "xi", "coi", "zoi")
+
+  # random parameters
+  random_parameters <- c("random", "zero_inflated_random")
+
+  # location parameters
+  location_parameters <- if (effects == "fixed") {
+    setdiff(elements, c(auxiliary_parameters, random_parameters))
+  } else {
+    setdiff(elements, auxiliary_parameters)
+  }
+
+  # fixed pattern?
+  if (all(component == "location")) {
+    return(location_parameters)
+  }
+
+  # fixed pattern?
+  if (all(component %in% c("distributional", "auxiliary"))) {
+    return(auxiliary_parameters)
+  }
+
 
   elements <- switch(
     effects,
     all = elements,
-    fixed = elements[elements %in% c("conditional", "conditional2", "conditional3", "precision", "zero_inflated", "dispersion", "instruments", "interactions", "simplex", "smooth_terms", "correlation", "slopes", "sigma", "nonlinear", "cluster", "extra", "scale")],
+    fixed = elements[elements %in% c("conditional", "conditional2", "conditional3", "precision", "zero_inflated", "dispersion", "instruments", "interactions", "simplex", "smooth_terms", "correlation", "slopes", "sigma", "nonlinear", "cluster", "extra", "scale", "marginal", "beta")],
     random = elements[elements %in% c("random", "zero_inflated_random")]
   )
 
@@ -303,6 +287,8 @@
     interactions = elements[elements == "interactions"],
     simplex = elements[elements == "simplex"],
     sigma = elements[elements == "sigma"],
+    beta = elements[elements == "beta"],
+    alpha = elements[elements == "alpha"],
     smooth_terms = elements[elements == "smooth_terms"],
     correlation = elements[elements == "correlation"],
     cluster = elements[elements == "cluster"],
@@ -310,7 +296,8 @@
     slopes = elements[elements == "slopes"],
     extra = elements[elements == "extra"],
     scale = elements[elements == "scale"],
-    precision = elements[elements == "precision"]
+    precision = elements[elements == "precision"],
+    marginal = elements[elements == "marginal"]
   )
 
   elements
@@ -404,6 +391,7 @@
     grepl("^(VGAM::s\\()", x, perl = TRUE) |
     grepl("^(mgcv::s\\()", x, perl = TRUE) |
     grepl("^(mgcv::ti\\()", x, perl = TRUE) |
+    grepl("^(mgcv::t2\\()", x, perl = TRUE) |
     grepl("^(mgcv::te\\()", x, perl = TRUE) |
     grepl("^(brms::s\\()", x, perl = TRUE) |
     grepl("^(brms::t2\\()", x, perl = TRUE) |
@@ -579,3 +567,236 @@
 
   dat[, 1:(ncol(dat) - 1), drop = FALSE]
 }
+
+
+
+#' @importFrom methods slot
+.is_baysian_emmeans <- function(x) {
+  if (inherits(x, "emm_list")) {
+    x <- x[[1]]
+  }
+  post.beta <- methods::slot(x, "post.beta")
+  !(all(dim(post.beta) == 1) && is.na(post.beta))
+}
+
+
+
+
+# safe conversion from factor to numeric
+#' @importFrom stats na.omit
+.factor_to_numeric <- function(x, lowest = NULL) {
+  if (is.data.frame(x)) {
+    as.data.frame(lapply(x, .factor_to_numeric_helper, lowest = lowest))
+  } else {
+    .factor_to_numeric_helper(x, lowest = lowest)
+  }
+}
+
+
+#' @importFrom stats na.omit
+.factor_to_numeric_helper <- function(x, lowest = NULL) {
+  if (is.numeric(x)) {
+    return(x)
+  }
+
+  if (is.logical(x)) {
+    return(as.numeric(x))
+  }
+
+  if (anyNA(suppressWarnings(as.numeric(as.character(stats::na.omit(x)))))) {
+    if (is.character(x)) {
+      x <- as.factor(x)
+    }
+    x <- droplevels(x)
+    levels(x) <- 1:nlevels(x)
+  }
+
+  out <- as.numeric(as.character(x))
+
+  if (!is.null(lowest)) {
+    difference <- min(out) - lowest
+    out <- out - difference
+  }
+
+  out
+}
+
+
+
+
+
+## copied from lme4::findbars() -----------------------
+
+
+.expandDoubleVert <- function(term) {
+  frml <- stats::formula(substitute(~x, list(x = term[[2]])))
+  newtrms <- paste0("0+", attr(terms(frml), "term.labels"))
+  if (attr(terms(frml), "intercept") != 0) {
+    newtrms <- c("1", newtrms)
+  }
+  stats::as.formula(paste("~(", paste(vapply(newtrms, function(trm) {
+    paste0(trm, "|", deparse(term[[3]]))
+  }, ""), collapse = ")+("), ")"))[[2]]
+}
+
+
+
+.expandDoubleVerts <- function(term) {
+  if (!is.name(term) && is.language(term)) {
+    if (term[[1]] == as.name("(")) {
+      term[[2]] <- .expandDoubleVerts(term[[2]])
+    }
+    stopifnot(is.call(term))
+    if (term[[1]] == as.name("||")) {
+      return(.expandDoubleVert(term))
+    }
+    term[[2]] <- .expandDoubleVerts(term[[2]])
+    if (length(term) != 2 && length(term) == 3) {
+      term[[3]] <- .expandDoubleVerts(term[[3]])
+    }
+  }
+  term
+}
+
+
+
+.findbars <- function(term) {
+  fb <- function(term) {
+    if (is.name(term) || !is.language(term)) {
+      return(NULL)
+    }
+    if (term[[1]] == as.name("(")) {
+      return(fb(term[[2]]))
+    }
+    stopifnot(is.call(term))
+    if (term[[1]] == as.name("|")) {
+      return(term)
+    }
+    if (length(term) == 2) {
+      return(fb(term[[2]]))
+    }
+    c(fb(term[[2]]), fb(term[[3]]))
+  }
+
+  expandSlash <- function(bb) {
+    makeInteraction <- function(x) {
+      if (length(x) < 2) {
+        return(x)
+      }
+      trm1 <- makeInteraction(x[[1]])
+      trm11 <- if (is.list(trm1)) {
+        trm1[[1]]
+      } else {
+        trm1
+      }
+      list(substitute(foo:bar, list(foo = x[[2]], bar = trm11)), trm1)
+    }
+    slashTerms <- function(x) {
+      if (!("/" %in% all.names(x))) {
+        return(x)
+      }
+      if (x[[1]] != as.name("/")) {
+        stop("unparseable formula for grouping factor", call. = FALSE)
+      }
+      list(slashTerms(x[[2]]), slashTerms(x[[3]]))
+    }
+    if (!is.list(bb)) {
+      expandSlash(list(bb))
+    } else {
+      unlist(lapply(bb, function(x) {
+        if (length(x) > 2 && is.list(trms <- slashTerms(x[[3]]))) {
+          lapply(unlist(makeInteraction(trms)), function(trm) {
+            substitute(foo | bar, list(foo = x[[2]], bar = trm))
+          })
+        } else {
+          x
+        }
+      }))
+    }
+  }
+  modterm <- .expandDoubleVerts(if (is(term, "formula")) {
+    term[[length(term)]]
+  } else {
+    term
+  })
+  expandSlash(fb(modterm))
+}
+
+
+
+
+## copied from lme4::nobars() -----------------------
+
+
+#' @importFrom methods is
+#' @importFrom stats reformulate
+.nobars <- function(term) {
+  nb <- .nobars_(term)
+  if (methods::is(term, "formula") && length(term) == 3 && is.symbol(nb)) {
+    nb <- stats::reformulate("1", response = deparse(nb))
+  }
+  if (is.null(nb)) {
+    nb <- if (methods::is(term, "formula")) {
+      ~1
+    } else {
+      1
+    }
+  }
+  nb
+}
+
+
+
+.nobars_ <- function(term) {
+  if (!(any(c("|", "||") %in% all.names(term)))) {
+    return(term)
+  }
+  if (.isBar(term)) {
+    return(NULL)
+  }
+  if (.isAnyArgBar(term)) {
+    return(NULL)
+  }
+  if (length(term) == 2) {
+    nb <- .nobars_(term[[2]])
+    if (is.null(nb)) {
+      return(NULL)
+    }
+    term[[2]] <- nb
+    return(term)
+  }
+  nb2 <- .nobars_(term[[2]])
+  nb3 <- .nobars_(term[[3]])
+  if (is.null(nb2)) {
+    return(nb3)
+  }
+  if (is.null(nb3)) {
+    return(nb2)
+  }
+  term[[2]] <- nb2
+  term[[3]] <- nb3
+  term
+}
+
+
+
+.isBar <- function(term) {
+  if (is.call(term)) {
+    if ((term[[1]] == as.name("|")) || (term[[1]] == as.name("||"))) {
+      return(TRUE)
+    }
+  }
+  FALSE
+}
+
+
+.isAnyArgBar <- function(term) {
+  if ((term[[1]] != as.name("~")) && (term[[1]] != as.name("("))) {
+    for (i in seq_along(term)) {
+      if (.isBar(term[[i]])) {
+        return(TRUE)
+      }
+    }
+  }
+  FALSE
+}
