@@ -41,20 +41,26 @@ supported_models <- function() {
     "afex_aov",
     "aov",
     "aovlist",
+    "Anova.mlm",
     "Arima",
     "averaging",
     "bamlss",
     "bamlss.frame",
     "bayesx",
+    "bayesQR",
     "BBmm",
     "BBreg",
     "bcplm",
+    "betamfx",
+    "betaor",
     "betareg",
     "BFBayesFactor",
+    "BGGM",
     "bife",
     "biglm",
     "bigglm",
     "blavaan",
+    "blrm",
     "bracl",
     "brglm",
     "brmsfit",
@@ -68,9 +74,12 @@ supported_models <- function() {
     "clmm",
     "clmm2",
     "clogit",
+    "coeftest",
     "complmrob",
     "coxme",
     "coxph",
+    "coxph.penal",
+    "coxr",
     "cpglm",
     "cpglmm",
     "crch",
@@ -80,6 +89,7 @@ supported_models <- function() {
     "feis",
     "felm",
     "feglm",
+    "fitdistr",
     "fixest",
     "flexsurvreg",
     "gam",
@@ -90,8 +100,11 @@ supported_models <- function() {
     "gbm",
     "gee",
     "geeglm",
+    "glht",
     "glimML",
+    "glmm",
     "glm",
+    "Glm",
     "glmmadmb",
     "glmmPQL",
     "glmmTMB",
@@ -100,10 +113,13 @@ supported_models <- function() {
     "glmx",
     "gls",
     "gmnl",
+    "HLfit",
     "htest",
     "hurdle",
+    "ivFixed",
     "iv_robust",
     "ivreg",
+    "ivprobit",
     "lavaan",
     "lm",
     "lm_robust",
@@ -112,37 +128,67 @@ supported_models <- function() {
     "lmRob",
     "lmerMod",
     "lmerModLmerTest",
+    "lmodel2",
+    "logitmfx",
+    "logitor",
     "logistf",
     "LORgee",
+    "lqm",
+    "lqmm",
     "lrm",
+    "manova",
     "MANOVA",
+    "margins",
     "maxLik",
     "mcmc",
     "MCMCglmm",
+    "mediate",
+    "metaplus",
     "merMod",
+    "merModList",
+    "mipo",
+    "mira",
     "mixed",
     "mixor",
     "MixMod",
+    "mle",
+    "mle2",
     "mlm",
     "mclogit",
+    "mcmc.list",
+    "meta_bma",
+    "meta_fixed",
+    "meta_random",
     "mlogit",
     "mmlogit",
     "multinom",
+    "negbinmfx",
+    "negbinirr",
     "ols",
+    "poissonmfx",
+    "poissonirr",
     "plm",
     "polr",
     "psm",
+    "probitmfx",
+    "ridgelm",
+    "riskRegression",
     "rlm",
     "rlmerMod",
     "RM",
     "rma",
     "rma.uni",
+    "robmixglm",
     "rq",
+    "rqs",
     "rqss",
+    "scam",
+    "sem",
     "speedlm",
     "speedglm",
     "stanmvreg",
     "stanreg",
+    "summary.lm",
     "survfit",
     "survreg",
     "svyglm",
